@@ -10,22 +10,31 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclPort.h,v 1.6.2.1 2003/04/16 23:31:46 dgp Exp $
+ * RCS: @(#) $Id: tclPort.h,v 1.15.10.3 2010/01/31 23:51:36 nijtmans Exp $
  */
 
 #ifndef _TCLPORT
 #define _TCLPORT
 
+#ifdef HAVE_TCL_CONFIG_H
+#include "tclConfig.h"
+#endif
+#if defined(_WIN32)
+#   include "tclWinPort.h"
+#endif
 #include "tcl.h"
+#if !defined(_WIN32)
+#   include "tclUnixPort.h"
+#endif
 
-#if defined(__WIN32__)
-#   include "../win/tclWinPort.h"
-#else
-#   if defined(MAC_TCL)
-#      include "tclMacPort.h"
-#   else
-#      include "tclUnixPort.h"
-#   endif
+#if defined(__CYGWIN__)
+#   define USE_PUTENV 1
+#   define USE_PUTENV_FOR_UNSET 1
+/* On Cygwin, the environment is imported from the Cygwin DLL. */
+    DLLIMPORT extern char **__cygwin_environ;
+    DLLIMPORT extern int cygwin_conv_to_win32_path(const char *, char *);
+#   define environ __cygwin_environ
+#   define timezone _timezone
 #endif
 
 #if !defined(LLONG_MIN)
